/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <getopt.h>
#include <unistd.h>

#include "aubio.h"	

#include "utils.h"
#include "cycle.h"

const char * output_filename = NULL;
const char * input_filename = NULL;

int verbose = 0;
int usejack = 0;

int main (int argc, char **argv) {  
	int i;
	/* settings */
	int buffer_size = 512;
	int overlap_size = 128;
	int channels = 2, frames;
	// speed measurements
	ticks t0,t1,t2,t3,t4,t5; // from cycle.h
	double time1, time2, time3, time4, time5;


	/* file objects */
	aubio_file_t * file;
	aubio_file_t * fileout;

	/* phase vocoder */
	aubio_pvoc_t * pv;
	
	/* onset detectors */
	onsetdetector_t *o1,*o2,*o3, *o4, *o5;
	fvec_t *onset1,*onset2,*onset3, *onset4, *onset5;
	fvec_t *ibuf, *ibuf2;
	fvec_t *obuf, *obuf2;
	cvec_t *fftgrain;

	int resample = 8;
	int rwsize = overlap_size*resample;
	// undersample factor 8
	aubio_resampler_t *downsample = new_aubio_resampler(1.0/(float)resample,1);
	// undersample factor 2
	aubio_resampler_t *upsample = new_aubio_resampler((float)resample,1);
	/* scale results */
	scale_t *scale1, *scale2, *scale3, *scale4, *scale5;

	// parsing command line argument
	parse_args(argc, argv);

	debug("Opening  files ...\n");
	file = new_file_ro (input_filename);
	file_info(file);
	channels = file_channels(file);
	fileout = new_file_wo(file, output_filename);
		
	debug("Phase voc init ... \n");
	pv = new_aubio_pvoc(buffer_size, overlap_size, channels);
	
	ibuf = new_fvec(rwsize, channels);
	obuf = new_fvec(rwsize, channels);
	ibuf2 = new_fvec(overlap_size, channels);
	obuf2 = new_fvec(overlap_size, channels);
	fftgrain = new_cvec(buffer_size, channels);
	
	/* onset init */
	o1 = new_onsetdetection(energy,				buffer_size,channels);
	o2 = new_onsetdetection(hfc,						buffer_size,channels);
	o3 = new_onsetdetection(complexdomain,	buffer_size,channels);
	o4 = new_onsetdetection(phase,					buffer_size,channels);
	o5 = new_onsetdetection(specdiff,			buffer_size,channels);
	onset1 = new_fvec(1, channels);
	onset2 = new_fvec(1, channels);
	onset3 = new_fvec(1, channels);
	onset4 = new_fvec(1, channels);
	onset5 = new_fvec(1, channels);

	scale1 = scale_new(0,15000.,		0.,1.);
	scale2 = scale_new(0,300000.,		0.,1.);
	scale3 = scale_new(0.,500.,			0.,1.);
	scale4 = scale_new(0.,110.,			0.,1.);
	scale5 = scale_new(0.,9.,				0.,1.);

	time1 = time2 = time3 = time4 = time5 = 0;
	t0 = t1 = t2 = t3 = t4 = t5 = 0;

	debug("Processing ... \n");
	/* reading and processing data from file */
	frames = 0;
	while ((rwsize == file_read(file, rwsize, ibuf) )) {
		aubio_resampler_process(downsample,ibuf,ibuf2);
		aubio_pvoc_do(pv, ibuf2, fftgrain);
		if (frames == 24) 
		for(i=0;i<fftgrain->length;i++) {
			debug("phas %d: %f\n",frames,fftgrain->phas[0][i]);
			debug("norm %d: %f\n",frames,fftgrain->norm[0][i]);
		}
		debug("end\n");
		/* complex onsets */
		t0 = getticks();
		onsetdetection(o1, fftgrain, onset1);
		t1 = getticks();
		onsetdetection(o2, fftgrain, onset2);
		t2 = getticks();
		onsetdetection(o3, fftgrain, onset3);
		t3 = getticks();
		onsetdetection(o4, fftgrain, onset4);
		t4 = getticks();
		onsetdetection(o5, fftgrain, onset5);
		t5 = getticks();

		time1 += elapsed(t1, t0);
		time2 += elapsed(t2, t1);
		time3 += elapsed(t3, t2);
		time4 += elapsed(t4, t3);
		time5 += elapsed(t5, t4);

		errmsg("1: %f\n",onset1->data[0][0]);
		errmsg("2: %f\n",onset2->data[0][0]);
		errmsg("3: %f\n",onset3->data[0][0]);
		errmsg("4: %f\n",onset4->data[0][0]);
		errmsg("5: %f\n",onset5->data[0][0]);
		
		aubio_pvoc_rdo(pv, fftgrain, obuf2);
		aubio_resampler_process(upsample,obuf2,obuf);

		// write output to file (but not the first frame)
		if (output_filename != NULL)
			file_write(fileout,rwsize,obuf);
		frames++;
	}
	debug("Processed %d frames of %d samples (%d frames).\n", 
			frames, buffer_size, frames*overlap_size);
	debug("The file was %d samples.\n", frames*overlap_size*resample);
	
	time1 /= frames;	debug("1: %f\n",time1);
	time2 /= frames;	debug("2: %f\n",time2);
	time3 /= frames;	debug("3: %f\n",time3);
	time4 /= frames;	debug("4: %f\n",time4);
	time5 /= frames;	debug("5: %f\n",time5);

	del_file(file);
	del_file(fileout);

	del_aubio_pvoc(pv);

	debug("End of program.\n");
	fflush(stderr);

	return 0;
}

