/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 */

#include "aubio_priv.h"
#include "hist.h"
#include "scale.h"
#include "mathutils.h" //vec_min vec_max

/********
 * Object Structure
 */

struct _hist_t {
	/*bug: move to a fvec */
	float ** hist;
	int nelems;
	int channels;
	float * cent;
	scale_t *scaler;
};


/********************************************************
 * Object Memory Allocation
 */

static hist_t * hist_malloc(int channels, int nelems);
static void hist_free(hist_t * s);

hist_t * hist_malloc(int channels, int nelems) {
	unsigned int i;
	hist_t * s = AUBIO_NEW(hist_t);
	s->channels = channels;
	s->nelems = nelems;
	
	s->hist = AUBIO_ARRAY(float*, channels);
	for (i=0; i< s->channels; i++) {
		s->hist[i] = AUBIO_ARRAY(float, nelems);
	}
	s->cent = AUBIO_ARRAY(float, channels);
	return s;
}

void hist_free(hist_t * s) {
	AUBIO_FREE(s);
}

/***
 * Object creation/deletion calls
 */
hist_t * hist_new (float ilow, float ihig, int nelems, int channels){
	float step = (ihig-ilow)/(float)(nelems);
	unsigned int i;
	hist_t * s = hist_malloc(channels, nelems);
	
	/* use scale to map ilow/ihig -> 0/nelems */
	s->scaler = scale_new(ilow,ihig,0,nelems);
	/* calculate centers now once */
	s->cent[0] = ilow + 0.5f * step;
	for (i=0; i < s->nelems; i++)
		s->cent[i] = s->cent[0] + i * step;
	
	return s;	
}

void hist_delete(hist_t *s) {
	hist_free(s);
}

/***
 * do it
 */
void hist_do (hist_t *s, fvec_t *input) 
{
	unsigned int i,j;
	scale_do(s->scaler, input);
	/* reset data */
	for (i=0; i < s->channels; i++)
		for (j=0; j < s->nelems; j++) 
			s->hist[i][j] = 0;
	/* run accum */
	for (i=0; i < input->channels; i++)
		for (j=0;  j < input->length; j++) 
				s->hist[i][(int)floor(input->data[i][j])] += 1;
}

void hist_do_notnull (hist_t *s, fvec_t *input) 
{
	unsigned int i,j;
	scale_do(s->scaler, input);
	/* reset data */
	for (i=0; i < s->channels; i++)
		for (j=0; j < s->nelems; j++) 
			s->hist[i][j] = 0;
	/* run accum */
	for (i=0; i < input->channels; i++)
		for (j=0;  j < input->length; j++) 
			if (input->data[i][j]!=0.0f) //input is not 0
				s->hist[i][(int)floor(input->data[i][j])] += 1;
}


void hist_dyn_notnull (hist_t *s, fvec_t *input) 
{
	unsigned int i,j;
	float ilow = vec_min(input);
	float ihig = vec_max(input);
	float step = (ihig-ilow)/(float)(s->nelems);
	
	/* readapt */
	scale_set(s->scaler, ilow, ihig, 0, s->nelems);

	/* recalculate centers */
	s->cent[0] = ilow + 0.5f * step;
	for (i=1; i < s->nelems; i++)
		s->cent[i] = s->cent[0] + i * step;

	/* scale */	
	scale_do(s->scaler, input);

	/* reset data */
	for (i=0; i < s->channels; i++)
		for (j=0; j < s->nelems; j++) 
			s->hist[i][j] = 0;
	/* run accum */
	for (i=0; i < input->channels; i++)
		for (j=0;  j < input->length; j++) 
			if (input->data[i][j]!=0.) //input was not 0
				s->hist[i][(int)floorf(input->data[i][j])] += 1;
}

void hist_weigth (hist_t *s) 
{
	unsigned int i,j;
	for (i=0; i < s->channels; i++)
		for (j=0; j < s->nelems; j++) {
			s->hist[i][j] *= s->cent[j];
		}
}

float hist_mean (hist_t *s) 
{
	unsigned int i,j;
	float tmp = 0.0f;
	for (i=0; i < s->channels; i++)
		for (j=0; j < s->nelems; j++)
			tmp += s->hist[i][j];
	return tmp/(float)(s->nelems);
}

