/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 
*/

#include "aubio_priv.h"
#include "mathutils.h"
#include "hist.h"
#include "onsetdetection.h"

struct _onsetdetector_t {
	onsetdetection_t type;
	/** Pointer to onsetdetection_<type> function */
	void (*funcpointer)(onsetdetector_t *o,
		cvec_t * fftgrain, fvec_t * onset);
	float threshold;
	fvec_t *oldmag;
	fft_data_t *meas	;
	fvec_t *dev1 ;
	fvec_t *theta ;
	fvec_t *theta1;
	fvec_t *theta2;

	hist_t * histog;
	
};


static onsetdetector_t * onsetdetection_alloc(onsetdetection_t type, int size, int channels);

/* Energy based onset detection function */
void onsetdetection_energy  (onsetdetector_t *o,
		cvec_t * fftgrain, fvec_t * onset) {
	unsigned int i,j;
	for (i=0;i<fftgrain->channels;i++) {
		onset->data[i][0] = 0.;
		for (j=0;j<fftgrain->length/2;j++) {
			onset->data[i][0] += SQR(fftgrain->norm[i][j]);
		}
	}
}

/* High Frequency Content onset detection function */
void onsetdetection_hfc(onsetdetector_t *o,	cvec_t * fftgrain, fvec_t * onset){
	unsigned int i,j;
	for (i=0;i<fftgrain->channels;i++) {
		onset->data[i][0] = 0.;
		for (j=0;j<fftgrain->length/2;j++) {
			onset->data[i][0] += (j+1)*fftgrain->norm[i][j];
		}
	}
}


/* Complex Domain Method onset detection function */
void onsetdetection_complex (onsetdetector_t *o, cvec_t * fftgrain, fvec_t * onset) {
	unsigned int i, j;
	unsigned int nbins = fftgrain->length;
	for (i=0;i<fftgrain->channels; i++)	{
		onset->data[i][0] = 0.;
		for (j=0;j<nbins; j++)	{
			o->dev1->data[i][j] 	 = unwrap2pi(
					fftgrain->phas[i][j]
					-2.0*o->theta1->data[i][j]+
					o->theta2->data[i][j]);
			o->meas[j] = fftgrain->norm[i][j]*CEXPC(I*o->dev1->data[i][j]);
			/* sum on all bins */
			onset->data[i][0]		 += //(fftgrain->norm[i][j]);
					SQRT(SQR( REAL(o->oldmag->data[i][j]-o->meas[j]) )
						+  SQR( IMAG(o->oldmag->data[i][j]-o->meas[j]) )
						);
			/* swap old phase data (need to remember 2 frames behind)*/
			o->theta2->data[i][j] = o->theta1->data[i][j];
			o->theta1->data[i][j] = fftgrain->phas[i][j];
			/* swap old magnitude data (1 frame is enough) */
			o->oldmag->data[i][j] = fftgrain->norm[i][j];
		}
	}
}


/* Phase Based Method onset detection function */
void onsetdetection_phase   (onsetdetector_t *o, cvec_t * fftgrain, fvec_t * onset){
	unsigned int i, j;
	unsigned int nbins = fftgrain->length;
	for (i=0;i<fftgrain->channels; i++)	{
		onset->data[i][0] = 0.0f;
		o->dev1->data[i][0]=0.;
		for ( j=0;j<nbins/2; j++ )	{
			o->dev1->data[i][j] = 
				unwrap2pi(
						fftgrain->phas[i][j]
						-2.0*o->theta1->data[i][j]
						+o->theta2->data[i][j]);
			if ( o->threshold < fftgrain->norm[i][j] )
				o->dev1->data[i][j] = ABS(o->dev1->data[i][j]);
			else 
				o->dev1->data[i][j] = 0.;
			/* keep a track of the past frames */
			o->theta2->data[i][j] = o->theta1->data[i][j];
			o->theta1->data[i][j] = fftgrain->phas[i][j];
		}
		/* apply o->histogram */
		hist_dyn_notnull(o->histog,o->dev1);
		/* weight it */
		hist_weigth(o->histog);
		/* its mean is the result */
		onset->data[i][0] = hist_mean(o->histog);	
		//onset->data[i][0] = vec_mean(o->dev1);
	}
}

/* Spectral difference method onset detection function */
void onsetdetection_specdiff(onsetdetector_t *o,
		cvec_t * fftgrain, fvec_t * onset){
	unsigned int i, j;
	unsigned int nbins = fftgrain->length;
	for (i=0;i<fftgrain->channels; i++)	{
		onset->data[i][0] = 0.0f;
		for (j=0;j<nbins; j++)	{
			o->dev1->data[i][j] = SQRT(
					ABS(SQR( fftgrain->norm[i][j])
						- SQR(o->oldmag->data[i][j])));
			if (o->threshold < fftgrain->norm[i][j] )
				o->dev1->data[i][j] = ABS(o->dev1->data[i][j]);
			else 
				o->dev1->data[i][j] = 0.0f;
			o->oldmag->data[i][j] = fftgrain->norm[i][j];
		}
		/* apply o->histogram (act as a low pass)*/
		hist_dyn_notnull(o->histog,o->dev1);
		/* weight it */
		hist_weigth(o->histog);
		/* its mean is the result */
		onset->data[i][0] = hist_mean(o->histog);	
	}
}

/* Generic function pointing to the choosen one */
void onsetdetection (onsetdetector_t *o, cvec_t * fftgrain, fvec_t * onset) {
	o->funcpointer(o,fftgrain,onset);
}

/* Allocate memory for an onset detection */
onsetdetector_t * new_onsetdetection	(onsetdetection_t type, int size, int channels){
	return onsetdetection_alloc(type,size,channels);
}

/* depending on the choosen type, allocate memory as needed */
onsetdetector_t * onsetdetection_alloc(onsetdetection_t type, int size, int channels){
	onsetdetector_t * o = AUBIO_NEW(onsetdetector_t);
	switch(type) {
		/* for both energy and hfc, o->norm is required */
		case energy: 
			break;
		case hfc:
			break;
		case complexdomain:
			o->oldmag = new_fvec(size,channels);
			//o->meas	 = new_fvec(size,channels);
			o->meas = AUBIO_ARRAY(fft_data_t,size);
			o->dev1	 = new_fvec(size,channels);
			o->theta1 = new_fvec(size,channels);
			o->theta2 = new_fvec(size,channels);
			break;
		case phase:
			o->dev1	 = new_fvec(size,channels);
			o->theta1 = new_fvec(size,channels);
			o->theta2 = new_fvec(size,channels);
			o->histog = hist_new(0.0f, PI, 10, channels);
			o->threshold = 0.1;
			break;
		case specdiff:
			o->oldmag = new_fvec(size,channels);
			o->dev1	  = new_fvec(size,channels);
			o->histog = hist_new(0.0f, PI, 10, channels);
			o->threshold = 0.1;
			break;
		default:
			break;
	}
	// this switch will be separated to be able to change 
	// between functions on the fly 
	switch(type) {
		case energy:
			o->funcpointer = onsetdetection_energy;
			break;
		case hfc:
			o->funcpointer = onsetdetection_hfc;
			break;
		case complexdomain:
			o->funcpointer = onsetdetection_complex;
			break;
		case phase:
			o->funcpointer = onsetdetection_phase;
			break;
		case specdiff:
			o->funcpointer = onsetdetection_specdiff;
			break;
		default:
			break;
	}
	o->type = type;
	return o;
}

void onsetdetection_free (onsetdetector_t *o){
	AUBIO_FREE(o);
}
