/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 
*/

/** @file
 *
 * Onset detection functions
 *
 * These functions are adapted from Juan Pablo Bello matlab code.
 *
 * - all of the following onset detection function take as arguments the fft of
 *   a windowed signal ( be created with an aubio_pvoc).
 *
 *
 * (the phasevocoder implementation does implement an fftshift like)
 *
 * - they output one float per frame and per channel (stored in a fvec_t * of
 *   size [channels][1])
 *
 *  Some of the functions should be improved by - downsampling the input of the
 *  phasevocoder - oversampling the ouput 
 */


#ifndef ONSETDETECTION_H
#define ONSETDETECTION_H

#include "sample.h"

/** onsetdetection types */
typedef enum {
	/** energy based */          
	energy,					
	/** spectral diff */         
	specdiff,
	/** high frequency content */
	hfc,						
	/** complex domain */        
	complexdomain,  
	/** phase fast */            
	phase					
} onsetdetection_t;

/** onsetdetection object */
typedef struct _onsetdetector_t onsetdetector_t;


/** Energy based onset detection function 
 *
 * calculates the local energy profile
 *
 * 	- buffer 1024
 * 	- overlap 512
 */
void onsetdetection_energy  (onsetdetector_t *o, cvec_t * fftgrain, fvec_t * onset);

/** High Frequency Content onset detection function
 *
 * 	- buffer 1024
 * 	- overlap 512
 */
void onsetdetection_hfc 	(onsetdetector_t *o, cvec_t * fftgrain, fvec_t * onset);

/** Complex Domain Method onset detection function 
 * 
 * 	From C. Duxbury & J. Pablo Bello
 * 		
 * 	- buffer 512
 * 	- overlap 128
 * 	- dowfact 8
 * 	- interpfact 2
 */
void onsetdetection_complex (onsetdetector_t *o, cvec_t * fftgrain, fvec_t * onset);

/** Phase Based Method onset detection function 
 *
 * 	- buffer 512
 * 	- overlap 128
 * 	- dowfact 8
 * 	- interpfact 2
 */
void onsetdetection_phase   (onsetdetector_t *o, cvec_t * fftgrain, fvec_t * onset);

/** Spectral difference method onset detection function 
 *
 * 	- buffer 512
 * 	- overlap 128
 * 	- dowfact 8
 * 	- interpfact 2
 */
void onsetdetection_specdiff(onsetdetector_t *o, cvec_t * fftgrain, fvec_t * onset);


/** Generic function pointing to the choosen one */
void onsetdetection(onsetdetector_t *o, cvec_t * fftgrain, fvec_t * onset);

/** Allocate memory for an onset detection */
onsetdetector_t * new_onsetdetection(onsetdetection_t type, int size, int channels);

/** Free memory for an onset detection */
void onsetdetection_free(onsetdetector_t *o);

#endif /* ONSETDETECTION_H */
